@@ -1,13 +1,13 @@
 /* mpfr_gamma -- gamma function
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
 The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MPFR Library is distributed in the hope that it will be useful, but
@@ -16,9 +16,9 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #define MPFR_NEED_LONGLONG_H
 #include "mpfr-impl.h"
@@ -28,7 +28,7 @@ MA 02110-1301, USA. */
 #undef IS_GAMMA
 
 /* return a sufficient precision such that 2-x is exact, assuming x < 0 */
-static mp_prec_t
+static mpfr_prec_t
 mpfr_gamma_2_minus_x_exact (mpfr_srcptr x)
 {
   /* Since x < 0, 2-x = 2+y with y := -x.
@@ -45,7 +45,7 @@ mpfr_gamma_2_minus_x_exact (mpfr_srcptr x)
 }
 
 /* return a sufficient precision such that 1-x is exact, assuming x < 1 */
-static mp_prec_t
+static mpfr_prec_t
 mpfr_gamma_1_minus_x_exact (mpfr_srcptr x)
 {
   if (MPFR_IS_POS(x))
@@ -72,16 +72,16 @@ bits_fac (unsigned long n)
   unsigned long r, k;
   mpfr_init2 (x, 38);
   mpfr_init2 (y, 38);
-  mpfr_set_ui (x, n, GMP_RNDZ);
+  mpfr_set_ui (x, n, MPFR_RNDZ);
   mpfr_set_str_binary (y, "10.101101111110000101010001011000101001"); /* upper bound of e */
-  mpfr_div (x, x, y, GMP_RNDZ);
-  mpfr_pow_ui (x, x, n, GMP_RNDZ);
-  mpfr_const_pi (y, GMP_RNDZ);
-  mpfr_mul_ui (y, y, 2 * n, GMP_RNDZ);
-  mpfr_sqrt (y, y, GMP_RNDZ);
-  mpfr_mul (x, x, y, GMP_RNDZ);
-  mpfr_log2 (x, x, GMP_RNDZ);
-  r = mpfr_get_ui (x, GMP_RNDU);
+  mpfr_div (x, x, y, MPFR_RNDZ);
+  mpfr_pow_ui (x, x, n, MPFR_RNDZ);
+  mpfr_const_pi (y, MPFR_RNDZ);
+  mpfr_mul_ui (y, y, 2 * n, MPFR_RNDZ);
+  mpfr_sqrt (y, y, MPFR_RNDZ);
+  mpfr_mul (x, x, y, MPFR_RNDZ);
+  mpfr_log2 (x, x, MPFR_RNDZ);
+  r = mpfr_get_ui (x, MPFR_RNDU);
   for (k = 2; k <= n; k *= 2)
     r -= n / k;
   mpfr_clear (x);
@@ -95,11 +95,11 @@ bits_fac (unsigned long n)
   i.e. with x = 1-t, then Gamma(x) = -Pi*(1-x)/sin(Pi*(2-x))/GAMMA(2-x)
 */
 int
-mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
+mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
 {
   mpfr_t xp, GammaTrial, tmp, tmp2;
   mpz_t fact;
-  mp_prec_t realprec;
+  mpfr_prec_t realprec;
   int compared, inex, is_integer;
   MPFR_GROUP_DECL (group);
   MPFR_SAVE_EXPO_DECL (expo);
@@ -160,34 +160,49 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
           If x < 2^E, then y > 2^(-E), thus ufp(y) > 2^(-E-1).
           A sufficient condition is thus EXP(x) + 2 <= -2 MAX(PREC(x),PREC(Y)).
   */
-  if (MPFR_EXP(x) + 2 <= -2 * (mp_exp_t) MAX(MPFR_PREC(x), MPFR_PREC(gamma)))
+  if (MPFR_GET_EXP (x) + 2
+      <= -2 * (mpfr_exp_t) MAX(MPFR_PREC(x), MPFR_PREC(gamma)))
     {
-      int positive = MPFR_IS_POS (x);
-      inex = mpfr_ui_div (gamma, 1, x, rnd_mode);
+      int sign = MPFR_SIGN (x); /* retrieve sign before possible override */
+      int special;
+      MPFR_BLOCK_DECL (flags);
+
+      MPFR_SAVE_EXPO_MARK (expo);
+
+      /* for overflow cases, see below; this needs to be done
+         before x possibly gets overridden. */
+      special =
+        MPFR_GET_EXP (x) == 1 - MPFR_EMAX_MAX &&
+        MPFR_IS_POS_SIGN (sign) &&
+        MPFR_IS_LIKE_RNDD (rnd_mode, sign) &&
+        mpfr_powerof2_raw (x);
+
+      MPFR_BLOCK (flags, inex = mpfr_ui_div (gamma, 1, x, rnd_mode));
       if (inex == 0) /* x is a power of two */
         {
-          if (positive)
+          /* return RND(1/x - euler) = RND(+/- 2^k - eps) with eps > 0 */
+          if (rnd_mode == MPFR_RNDN || MPFR_IS_LIKE_RNDU (rnd_mode, sign))
+            inex = 1;
+          else
             {
-              if (rnd_mode == GMP_RNDU || rnd_mode == GMP_RNDN)
-                inex = 1;
-              else /* round to zero or to -Inf */
-                {
-                  mpfr_nextbelow (gamma); /* 2^k - epsilon */
-                  inex = -1;
-                }
-            }
-          else /* negative */
-            {
-              if (rnd_mode == GMP_RNDU || rnd_mode == GMP_RNDZ)
-                {
-                  mpfr_nextabove (gamma); /* -2^k + epsilon */
-                  inex = 1;
-                }
-              else /* round to nearest and to -Inf */
-                inex = -1;
+              mpfr_nextbelow (gamma);
+              inex = -1;
             }
         }
-      return inex;
+      else if (MPFR_UNLIKELY (MPFR_OVERFLOW (flags)))
+        {
+          /* Overflow in the division 1/x. This is a real overflow, except
+             in RNDZ or RNDD when 1/x = 2^emax, i.e. x = 2^(-emax): due to
+             the "- euler", the rounded value in unbounded exponent range
+             is 0.111...11 * 2^emax (not an overflow). */
+          if (!special)
+            MPFR_SAVE_EXPO_UPDATE_FLAGS (expo, flags);
+        }
+      MPFR_SAVE_EXPO_FREE (expo);
+      /* Note: an overflow is possible with an infinite result;
+         in this case, the overflow flag will automatically be
+         restored by mpfr_check_range. */
+      return mpfr_check_range (gamma, inex, rnd_mode);
     }
 
   is_integer = mpfr_integer_p (x);
@@ -210,12 +225,12 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
      Warning: since the generic code does not handle exact cases,
      we want all cases where gamma(x) is exact to be treated here.
   */
-  if (is_integer && mpfr_fits_ulong_p (x, GMP_RNDN))
+  if (is_integer && mpfr_fits_ulong_p (x, MPFR_RNDN))
     {
       unsigned long int u;
-      mp_prec_t p = MPFR_PREC(gamma);
-      u = mpfr_get_ui (x, GMP_RNDN);
-      if (u < 44787929UL && bits_fac (u - 1) <= p + (rnd_mode == GMP_RNDN))
+      mpfr_prec_t p = MPFR_PREC(gamma);
+      u = mpfr_get_ui (x, MPFR_RNDN);
+      if (u < 44787929UL && bits_fac (u - 1) <= p + (rnd_mode == MPFR_RNDN))
         /* bits_fac: lower bound on the number of bits of m,
            where gamma(x) = (u-1)! = m*2^e with m odd. */
         return mpfr_fac_ui (gamma, u - 1, rnd_mode);
@@ -225,12 +240,15 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
          to return a mpz_t or mpfr_t. */
     }
 
+  MPFR_SAVE_EXPO_MARK (expo);
+
   /* check for overflow: according to (6.1.37) in Abramowitz & Stegun,
      gamma(x) >= exp(-x) * x^(x-1/2) * sqrt(2*Pi)
               >= 2 * (x/e)^x / x for x >= 1 */
   if (compared > 0)
     {
       mpfr_t yp;
+      mpfr_exp_t expxp;
       MPFR_BLOCK_DECL (flags);
 
       /* 1/e rounded down to 53 bits */
@@ -238,24 +256,25 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
       mpfr_init2 (xp, 53);
       mpfr_init2 (yp, 53);
       mpfr_set_str_binary (xp, EXPM1_STR);
-      mpfr_mul (xp, x, xp, GMP_RNDZ);
-      mpfr_sub_ui (yp, x, 2, GMP_RNDZ);
-      mpfr_pow (xp, xp, yp, GMP_RNDZ); /* (x/e)^(x-2) */
+      mpfr_mul (xp, x, xp, MPFR_RNDZ);
+      mpfr_sub_ui (yp, x, 2, MPFR_RNDZ);
+      mpfr_pow (xp, xp, yp, MPFR_RNDZ); /* (x/e)^(x-2) */
       mpfr_set_str_binary (yp, EXPM1_STR);
-      mpfr_mul (xp, xp, yp, GMP_RNDZ); /* x^(x-2) / e^(x-1) */
-      mpfr_mul (xp, xp, yp, GMP_RNDZ); /* x^(x-2) / e^x */
-      mpfr_mul (xp, xp, x, GMP_RNDZ); /* lower bound on x^(x-1) / e^x */
-      MPFR_BLOCK (flags, mpfr_mul_2ui (xp, xp, 1, GMP_RNDZ));
+      mpfr_mul (xp, xp, yp, MPFR_RNDZ); /* x^(x-2) / e^(x-1) */
+      mpfr_mul (xp, xp, yp, MPFR_RNDZ); /* x^(x-2) / e^x */
+      mpfr_mul (xp, xp, x, MPFR_RNDZ); /* lower bound on x^(x-1) / e^x */
+      MPFR_BLOCK (flags, mpfr_mul_2ui (xp, xp, 1, MPFR_RNDZ));
+      expxp = MPFR_GET_EXP (xp);
       mpfr_clear (xp);
       mpfr_clear (yp);
-      return MPFR_OVERFLOW (flags) ? mpfr_overflow (gamma, rnd_mode, 1)
-        : mpfr_gamma_aux (gamma, x, rnd_mode);
+      MPFR_SAVE_EXPO_FREE (expo);
+      return MPFR_OVERFLOW (flags) || expxp > __gmpfr_emax ?
+        mpfr_overflow (gamma, rnd_mode, 1) :
+        mpfr_gamma_aux (gamma, x, rnd_mode);
     }
 
   /* now compared < 0 */
 
-  MPFR_SAVE_EXPO_MARK (expo);
-
   /* check for underflow: for x < 1,
      gamma(x) = Pi*(x-1)/sin(Pi*(2-x))/gamma(2-x).
      Since gamma(2-x) >= 2 * ((2-x)/e)^(2-x) / (2-x), we have
@@ -266,17 +285,17 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
   if (MPFR_IS_NEG(x))
     {
       int underflow = 0, sgn, ck;
-      mp_prec_t w;
+      mpfr_prec_t w;
 
       mpfr_init2 (xp, 53);
       mpfr_init2 (tmp, 53);
       mpfr_init2 (tmp2, 53);
       /* we want an upper bound for x * [log(2-x)-1].
          since x < 0, we need a lower bound on log(2-x) */
-      mpfr_ui_sub (xp, 2, x, GMP_RNDD);
-      mpfr_log (xp, xp, GMP_RNDD);
-      mpfr_sub_ui (xp, xp, 1, GMP_RNDD);
-      mpfr_mul (xp, xp, x, GMP_RNDU);
+      mpfr_ui_sub (xp, 2, x, MPFR_RNDD);
+      mpfr_log2 (xp, xp, MPFR_RNDD);
+      mpfr_sub_ui (xp, xp, 1, MPFR_RNDD);
+      mpfr_mul (xp, xp, x, MPFR_RNDU);
 
       /* we need an upper bound on 1/|sin(Pi*(2-x))|,
          thus a lower bound on |sin(Pi*(2-x))|.
@@ -288,23 +307,23 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
       w += 17; /* to get tmp2 small enough */
       mpfr_set_prec (tmp, w);
       mpfr_set_prec (tmp2, w);
-      ck = mpfr_ui_sub (tmp, 2, x, GMP_RNDN);
+      ck = mpfr_ui_sub (tmp, 2, x, MPFR_RNDN);
       MPFR_ASSERTD (ck == 0);
-      mpfr_const_pi (tmp2, GMP_RNDN);
-      mpfr_mul (tmp2, tmp2, tmp, GMP_RNDN); /* Pi*(2-x) */
-      mpfr_sin (tmp, tmp2, GMP_RNDN); /* sin(Pi*(2-x)) */
+      mpfr_const_pi (tmp2, MPFR_RNDN);
+      mpfr_mul (tmp2, tmp2, tmp, MPFR_RNDN); /* Pi*(2-x) */
+      mpfr_sin (tmp, tmp2, MPFR_RNDN); /* sin(Pi*(2-x)) */
       sgn = mpfr_sgn (tmp);
-      mpfr_abs (tmp, tmp, GMP_RNDN);
-      mpfr_mul_ui (tmp2, tmp2, 3, GMP_RNDU); /* 3Pi(2-x) */
-      mpfr_add_ui (tmp2, tmp2, 1, GMP_RNDU); /* 3Pi(2-x)+1 */
-      mpfr_div_2ui (tmp2, tmp2, mpfr_get_prec (tmp), GMP_RNDU);
+      mpfr_abs (tmp, tmp, MPFR_RNDN);
+      mpfr_mul_ui (tmp2, tmp2, 3, MPFR_RNDU); /* 3Pi(2-x) */
+      mpfr_add_ui (tmp2, tmp2, 1, MPFR_RNDU); /* 3Pi(2-x)+1 */
+      mpfr_div_2ui (tmp2, tmp2, mpfr_get_prec (tmp), MPFR_RNDU);
       /* if tmp2<|tmp|, we get a lower bound */
       if (mpfr_cmp (tmp2, tmp) < 0)
         {
-          mpfr_sub (tmp, tmp, tmp2, GMP_RNDZ); /* low bnd on |sin(Pi*(2-x))| */
-          mpfr_ui_div (tmp, 12, tmp, GMP_RNDU); /* upper bound */
-          mpfr_log (tmp, tmp, GMP_RNDU);
-          mpfr_add (tmp, tmp, xp, GMP_RNDU);
+          mpfr_sub (tmp, tmp, tmp2, MPFR_RNDZ); /* low bnd on |sin(Pi*(2-x))| */
+          mpfr_ui_div (tmp, 12, tmp, MPFR_RNDU); /* upper bound */
+          mpfr_log2 (tmp, tmp, MPFR_RNDU);
+          mpfr_add (xp, tmp, xp, MPFR_RNDU);
           underflow = mpfr_cmp_si (xp, expo.saved_emin - 2) <= 0;
         }
 
@@ -314,14 +333,14 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
       if (underflow) /* the sign is the opposite of that of sin(Pi*(2-x)) */
         {
           MPFR_SAVE_EXPO_FREE (expo);
-          return mpfr_underflow (gamma, (rnd_mode == GMP_RNDN) ? GMP_RNDZ : rnd_mode, -sgn);
+          return mpfr_underflow (gamma, (rnd_mode == MPFR_RNDN) ? MPFR_RNDZ : rnd_mode, -sgn);
         }
     }
 
   realprec = MPFR_PREC (gamma);
   /* we want both 1-x and 2-x to be exact */
   {
-    mp_prec_t w;
+    mpfr_prec_t w;
     w = mpfr_gamma_1_minus_x_exact (x);
     if (realprec < w)
       realprec = w;
@@ -338,19 +357,19 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
   MPFR_ZIV_INIT (loop, realprec);
   for (;;)
     {
-      mp_exp_t err_g;
+      mpfr_exp_t err_g;
       int ck;
       MPFR_GROUP_REPREC_4 (group, realprec, xp, tmp, tmp2, GammaTrial);
 
       /* reflection formula: gamma(x) = Pi*(x-1)/sin(Pi*(2-x))/gamma(2-x) */
 
-      ck = mpfr_ui_sub (xp, 2, x, GMP_RNDN);
+      ck = mpfr_ui_sub (xp, 2, x, MPFR_RNDN);
       MPFR_ASSERTD(ck == 0); /* 2-x, exact */
-      mpfr_gamma (tmp, xp, GMP_RNDN);   /* gamma(2-x), error (1+u) */
-      mpfr_const_pi (tmp2, GMP_RNDN);   /* Pi, error (1+u) */
-      mpfr_mul (GammaTrial, tmp2, xp, GMP_RNDN); /* Pi*(2-x), error (1+u)^2 */
+      mpfr_gamma (tmp, xp, MPFR_RNDN);   /* gamma(2-x), error (1+u) */
+      mpfr_const_pi (tmp2, MPFR_RNDN);   /* Pi, error (1+u) */
+      mpfr_mul (GammaTrial, tmp2, xp, MPFR_RNDN); /* Pi*(2-x), error (1+u)^2 */
       err_g = MPFR_GET_EXP(GammaTrial);
-      mpfr_sin (GammaTrial, GammaTrial, GMP_RNDN); /* sin(Pi*(2-x)) */
+      mpfr_sin (GammaTrial, GammaTrial, MPFR_RNDN); /* sin(Pi*(2-x)) */
       err_g = err_g + 1 - MPFR_GET_EXP(GammaTrial);
       /* let g0 the true value of Pi*(2-x), g the computed value.
          We have g = g0 + h with |h| <= |(1+u^2)-1|*g.
@@ -358,17 +377,17 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
          The relative error is thus bounded by |(1+u^2)-1|*g/sin(g)
          <= |(1+u^2)-1|*2^err_g. <= 2.25*u*2^err_g for |u|<=1/4.
          With the rounding error, this gives (0.5 + 2.25*2^err_g)*u. */
-      ck = mpfr_sub_ui (xp, x, 1, GMP_RNDN);
+      ck = mpfr_sub_ui (xp, x, 1, MPFR_RNDN);
       MPFR_ASSERTD(ck == 0); /* x-1, exact */
-      mpfr_mul (xp, tmp2, xp, GMP_RNDN); /* Pi*(x-1), error (1+u)^2 */
-      mpfr_mul (GammaTrial, GammaTrial, tmp, GMP_RNDN);
+      mpfr_mul (xp, tmp2, xp, MPFR_RNDN); /* Pi*(x-1), error (1+u)^2 */
+      mpfr_mul (GammaTrial, GammaTrial, tmp, MPFR_RNDN);
       /* [1 + (0.5 + 2.25*2^err_g)*u]*(1+u)^2 = 1 + (2.5 + 2.25*2^err_g)*u
          + (0.5 + 2.25*2^err_g)*u*(2u+u^2) + u^2.
          For err_g <= realprec-2, we have (0.5 + 2.25*2^err_g)*u <=
          0.5*u + 2.25/4 <= 0.6875 and u^2 <= u/4, thus
          (0.5 + 2.25*2^err_g)*u*(2u+u^2) + u^2 <= 0.6875*(2u+u/4) + u/4
          <= 1.8*u, thus the rel. error is bounded by (4.5 + 2.25*2^err_g)*u. */
-      mpfr_div (GammaTrial, xp, GammaTrial, GMP_RNDN);
+      mpfr_div (GammaTrial, xp, GammaTrial, MPFR_RNDN);
       /* the error is of the form (1+u)^3/[1 + (4.5 + 2.25*2^err_g)*u].
          For realprec >= 5 and err_g <= realprec-2, [(4.5 + 2.25*2^err_g)*u]^2
          <= 0.71, and for |y|<=0.71, 1/(1-y) can be written 1+a*y with a<=4.
